@@ -12,6 +12,7 @@ from gantry.logger.main import (
     ping,
     setup_logger,
 )
+from gantry.utils import compute_feedback_id
 
 __all__ = [
     "init",
@@ -26,4 +27,5 @@ __all__ = [
     "log_prediction_event",
     "get_client",
     "setup_logger",
+    "compute_feedback_id",
 ]
