@@ -5,12 +5,9 @@ import logging
 import math
 import os
 import random
-import subprocess
 import sys
-import uuid
-from enum import Enum
 from pathlib import Path
-from typing import Any, Dict, Iterable, List, Optional, Tuple, Union, no_type_check
+from typing import Any, Dict, List, Optional, Union, no_type_check
 from warnings import warn
 
 import colorama
@@ -24,332 +21,34 @@ from gantry.exceptions import (
     GantryException,
     GantryLoggingException,
 )
-from gantry.logger.stores import APILogStore, BaseLogStore
+from gantry.logger.constants import CHUNK_SIZE, BatchTypes, Delimiter, UploadFileType
+from gantry.logger.event_builder import (
+    _build_batch_meta,
+    _build_feedback_event,
+    _build_prediction_and_feedback_events,
+    _build_prediction_event,
+    _build_prediction_events,
+    _create_timestamp_idx,
+    _enrich_events_with_batch_id,
+)
+from gantry.logger.stores import APILogStore
+from gantry.logger.utils import (
+    _batch_fail_msg,
+    _batch_success_msg,
+    _check_sample_rate,
+    _is_empty,
+    _is_not_empty,
+    _log_exception,
+    get_file_linecount,
+    progressbar,
+)
 from gantry.query.core.utils import check_response
 from gantry.serializers import serializable_value
-from gantry.utils import (
-    check_event_time_in_future,
-    compute_feedback_id,
-    format_msg_with_color,
-)
-from gantry.validators import validate_logged_field_datatype
+from gantry.utils import format_msg_with_color
 
 logger = logging.getLogger(__name__)
 
 
-class BatchTypes(Enum):
-    RECORD = "RECORD"
-    FEEDBACK = "FEEDBACK"
-    PREDICTION = "PREDICTION"
-
-
-def _batch_fail_msg(batch_id):
-    if batch_id:
-        logger.info(f"Batch with id: {batch_id} FAILED")
-
-
-def _batch_success_msg(batch_id, application, log_store: Union[APILogStore, BaseLogStore]):
-    if batch_id:
-        if isinstance(log_store, APILogStore):
-            host = log_store._api_client._host
-        else:
-            host = "[YOUR_GANTRY_DASHBOARD]"
-
-        logger.info("Track your batch at {}/applications/{}/batches".format(host, application))
-        logger.info("Look for batch id: {}".format(batch_id))
-
-
-def get_file_linecount(filepath):
-    return int(subprocess.check_output(["wc", "-l", filepath]).split()[0])
-
-
-def _build_prediction_and_feedback_events(
-    application: str,
-    context: Dict[str, str],
-    environment: str,
-    timestamp_idx: Iterable,
-    tags: dict,
-    inputs: Union[List[dict], pd.DataFrame],
-    outputs: Union[List[dict], List[Any], pd.DataFrame],
-    feedbacks: Union[List[dict], pd.DataFrame],
-    version: List[Union[str, int]] = None,
-    feedback_keys: Optional[List[str]] = None,
-    ignore_inputs: Optional[List[str]] = None,
-    sample_rate: float = 1.0,
-):
-    """Build prediction and feedback events for log_records"""
-    _check_sample_rate(sample_rate)
-
-    events = []
-    for idx, timestamp in timestamp_idx:
-        if random.random() > sample_rate:
-            continue
-
-        feedback_id = {k: inputs[idx][k] for k in (feedback_keys or [])}
-        version_ = version[idx] if version is not None else None  # type: ignore
-
-        event = {}
-
-        event.update(
-            _build_prediction_event(
-                context,
-                environment,
-                inputs[idx],
-                outputs[idx],
-                application,
-                version_,
-                feedback_keys,
-                ignore_inputs,
-                feedback_id=None,
-                custom_timestamp=timestamp,
-                tags=tags,
-            )
-        )
-
-        feedback_event = _build_feedback_event(
-            context,
-            application,
-            feedback_id,
-            feedbacks[idx],
-            version_,  # type: ignore
-            timestamp,
-        )
-
-        # Build an event with prediction data
-        # and feedback data, now that backend
-        # supports it.
-        event["metadata"].update(feedback_event.pop("metadata"))
-        event.update(feedback_event)
-
-        events.append(event)
-
-    return events
-
-
-def _check_sample_rate(sample_rate):
-    assert sample_rate <= 1 and sample_rate >= 0
-
-
-def _create_timestamp_idx(
-    sort_on_timestamp: bool, timestamps: Optional[Iterable[Any]], record_count: int
-) -> Iterable:
-    """
-    This function is used to take an index of timestamps, possibly None, and a length,
-    and turn it into a mapping from the current timestamp to the original index of the
-    corresponding value. The goal is to send all data to Gantry in timestamp order to
-    minimize summarization window computing overhead by packing events of a single
-    window into a single task.
-    """
-    if timestamps:
-        timestamp_idx: Iterable[Tuple[int, Any]] = enumerate(timestamps)
-        if sort_on_timestamp:
-            timestamp_idx = sorted(timestamp_idx, key=lambda el: el[1])
-    else:
-        timestamp = datetime.datetime.utcnow()
-        timestamp_idx = ((i, timestamp) for i in range(record_count))
-
-    return timestamp_idx
-
-
-def _enrich_events_with_batch_id(events, batch_id):
-    for event in events:
-        event["batch_id"] = batch_id
-
-
-def _build_feedback_event(
-    context: dict,
-    application: str,
-    feedback_id: dict,
-    feedback: dict,
-    feedback_version: Optional[int] = None,
-    timestamp: Optional[datetime.datetime] = None,
-    batch_id: Optional[str] = None,
-) -> dict:
-    """
-    Create a feedback event record for logging.
-    """
-    metadata: Dict[str, Any] = {}
-    metadata.update(context)
-    metadata.update({"func_name": application, "feedback_version": feedback_version})
-    computed_feedback_id = compute_feedback_id(feedback_id, list(feedback_id.keys()))
-
-    log_time = datetime.datetime.utcnow()
-    event_time = timestamp if timestamp else log_time
-    check_event_time_in_future(event_time)
-    log_time_str, event_time_str = log_time.isoformat(), event_time.isoformat()
-
-    validate_logged_field_datatype(feedback, paths=["feedback"])
-    validate_logged_field_datatype(feedback_id, paths=["feedback_id"])
-    validate_logged_field_datatype(event_time_str, paths=["timestamp"])
-
-    return {
-        "event_id": uuid.uuid4(),
-        "timestamp": event_time_str,
-        "log_timestamp": log_time_str,
-        "metadata": metadata,
-        "feedback_id_inputs": feedback_id,
-        "feedback": feedback,
-        "feedback_id": computed_feedback_id,
-        "batch_id": batch_id,
-    }
-
-
-def _build_prediction_events(
-    application: str,
-    inputs: Union[List[dict], pd.DataFrame],
-    outputs: Union[List[dict], List[Any], pd.DataFrame],
-    timestamp_idx: Iterable,
-    context: Dict[str, str],
-    environment: str,
-    tags: dict,
-    version: Optional[List[Union[str, int]]],
-    feedback_keys: Optional[List[str]] = None,
-    ignore_inputs: Optional[List[str]] = None,
-    feedback_ids: Optional[List[dict]] = None,
-    sample_rate: float = 1.0,
-    batch_id: str = None,
-):
-    _check_sample_rate(sample_rate)
-
-    events = []
-    for idx, timestamp in timestamp_idx:
-        if random.random() > sample_rate:
-            continue
-
-        feedback_id = feedback_ids[idx] if feedback_ids else None
-        version_ = version[idx] if version is not None else None
-
-        events.append(
-            _build_prediction_event(
-                context,
-                environment,
-                inputs[idx],
-                outputs[idx],
-                application,
-                version_,
-                feedback_keys,
-                ignore_inputs,
-                batch_id=batch_id,
-                feedback_id=feedback_id,
-                custom_timestamp=timestamp,
-                tags=tags,
-            )
-        )
-
-    return events
-
-
-def _build_prediction_event(
-    context: dict,
-    env: str,
-    inputs: Dict,
-    outputs: Any,
-    application: str,
-    version: Optional[Union[int, str]],
-    feedback_keys: Optional[List[str]],
-    ignore_inputs: Optional[List[str]],
-    batch_id: Optional[str] = None,
-    feedback_id: Optional[dict] = None,
-    tags: Optional[Dict[str, str]] = None,
-    custom_timestamp: Optional[datetime.datetime] = None,
-):
-    metadata = {}
-    metadata.update(context)
-    metadata.update(
-        {
-            "func_name": application,
-            "version": version,
-            "feedback_keys": feedback_keys,
-            "ignore_inputs": ignore_inputs,
-            "provided_feedback_id": feedback_id,
-        }
-    )
-    inputs = dict(inputs)  # make a copy so we can pop keys safely
-
-    if feedback_id:
-        if feedback_keys:
-            raise GantryLoggingException(
-                "Cannot specify feedback_id and feedback_keys at same time."
-            )
-        # Compute feedback_id just from provided feedback_id dictionary
-        computed_feedback_id = compute_feedback_id(feedback_id, list(feedback_id.keys()))
-    else:
-        computed_feedback_id = compute_feedback_id(inputs, feedback_keys)
-
-    if ignore_inputs:
-        for ignore in ignore_inputs:
-            inputs.pop(ignore, None)
-
-    log_time = datetime.datetime.utcnow()
-    event_time = custom_timestamp if custom_timestamp else log_time
-    check_event_time_in_future(event_time)
-    log_time_str, event_time_str = log_time.isoformat(), event_time.isoformat()
-
-    processed_tags = {"env": env}
-    if tags:
-        processed_tags.update(tags)
-
-    validate_logged_field_datatype(inputs, paths=["inputs"])
-    validate_logged_field_datatype(outputs, paths=["outputs"])
-
-    return {
-        "event_id": uuid.uuid4(),
-        "log_timestamp": log_time_str,
-        "timestamp": event_time_str,
-        "metadata": metadata,
-        "inputs": inputs,
-        "outputs": outputs,
-        "feedback_id": computed_feedback_id,
-        "tags": processed_tags,
-        "batch_id": batch_id,
-    }
-
-
-def _build_batch_meta(event_len: List[dict], type_: str) -> dict:
-    return {
-        "size": event_len,
-        "started_at": datetime.datetime.utcnow().isoformat(),
-        "type": type_,
-    }
-
-
-def _log_exception(func):
-    """Handles all exceptions thrown by func, and logs them to prevent the
-    func call from crashing.
-    """
-
-    @functools.wraps(func)
-    def wrapper(*args, **kwargs):
-        try:
-            return func(*args, **kwargs)
-        except GantryLoggingException as le:
-            # this is caused by a user error
-            # log the error without the stacktrace
-            logger.error("Error logging data to Gantry: %s", le)
-        except GantryBatchCreationException as e:
-            # Blocking exception for batch creation failure
-            raise e
-        except Exception as e:
-            # make sure our logging errors don't cause exceptions in main program
-            logger.exception("Internal exception in Gantry client: %s", e)
-
-    return wrapper
-
-
-def _is_empty(data: Union[List[Any], pd.DataFrame, Any] = None) -> bool:
-    """
-    Check whether input data is empty or not
-    Returns True if data is None or empty, False if data size >= 1
-    """
-    if isinstance(data, pd.DataFrame):
-        return data.empty
-    return True if not data else False
-
-
-def _is_not_empty(data: Union[List[Any], pd.DataFrame, Any] = None) -> bool:
-    return not _is_empty(data)
-
-
 class Gantry:
     def __init__(
         self,
@@ -503,41 +202,60 @@ class Gantry:
         outputs: List[str] = [],
         feedbacks: List[str] = [],
         metadata: List[str] = [],
+        feedback_id: List[str] = [],
+        feedback_keys: Optional[List[str]] = None,
     ):
+        # Get info for tracking batch during processing.
+        num_events = get_file_linecount(filepath) - 1  # Subtract the header line.
+        logger.info(f"Processing {num_events} lines.")
+
         # Read the headers of the file, infer and build linking datastruct.
         gantry_data_link: Dict[str, Any] = {
-            "file_type": "CSV_WITH_HEADERS",
+            "file_type": UploadFileType.CSV_WITH_HEADERS.value,
+            "num_events": num_events,
+            "application": application,
+            "version": version,
+            "log_timestamp": datetime.datetime.utcnow().isoformat(),
             "timestamp": dict(),
             "inputs": dict(),
             "outputs": dict(),
             "feedback": dict(),
-            "metadata": dict(),
+            "tags": {
+                "env": {"val": self.environment}
+            },  # This makes env.val exactly passed in rather than reference to a file column.
+            "feedback_id": dict(),
+            "feedback_keys": list(),
         }
         with open(filepath, "r") as f_in:
-            headers_line = f_in.readline().split(",")
+            headers_line = f_in.readline().split(Delimiter.COMMA.value)
         for col_index, raw_header in enumerate(headers_line):
             header = raw_header.strip()
             if header == timestamp or (not timestamp and header.startswith("timestamp")):
-                gantry_data_link["timestamp"] = {"ref": col_index}
+                gantry_data_link["timestamp"][header] = {"ref": col_index}
             elif header in outputs or header.startswith("output"):
                 gantry_data_link["outputs"][header] = {"ref": col_index}
             elif header in feedbacks or header.startswith("feedback"):
                 gantry_data_link["feedback"][header] = {"ref": col_index}
             elif header in metadata or header.startswith("metadata"):
-                gantry_data_link["metadata"][header] = {"ref": col_index}
-            else:
+                gantry_data_link["tags"][header] = {"ref": col_index}
+            elif len(inputs) == 0 or header in inputs or header.startswith("input"):
                 gantry_data_link["inputs"][header] = {"ref": col_index}
 
-        # Get info for tracking batch during processing.
-        line_count = get_file_linecount(filepath)
-        logger.info(f"Processing {line_count - 1} lines.")
+            if header in feedback_id or header == "feedback_id":
+                gantry_data_link["feedback_id"][header] = {"ref": col_index}
+            elif feedback_keys is not None and header in feedback_keys:
+                gantry_data_link["feedback_keys"].append(header)
+        if not gantry_data_link["tags"].get("env"):
+            raise GantryLoggingException(
+                "Please initialize gantry logger with 'environment' or specify an 'env' tag."
+            )
 
         # Calculate part counts and generate presigned s3 urls.
         path = Path(filepath)
-        CHUNK_SIZE = 10 * 1024 * 1024  # Chunk size 10 MB
         object_size = os.path.getsize(filepath)  # in bytes
         total_num_parts = math.ceil(object_size / CHUNK_SIZE)
 
+        logger.info("Initializing upload to Gantry")
         preupload_res = self.log_store._api_client.request(
             "GET",
             "/api/v1/ingest/file-preupload",
@@ -552,15 +270,19 @@ class Gantry:
         with open(filepath, "r") as f_in:
             block_read = functools.partial(f_in.read, CHUNK_SIZE)
             block_iterator = iter(block_read, "")
-            for part_num, block in enumerate(block_iterator):
+            part_num = 1  # AWS S3 needs part num to start with 1.
+            for block in progressbar(
+                block_iterator, prefix="Uploading to Gantry Bucket: ", count=total_num_parts
+            ):
                 # TODO: parallelize this client side?
-                response = requests.put(signed_urls[part_num], data=block)
+                response = requests.put(signed_urls[part_num - 1], data=block)
                 etag = str(response.headers["ETag"])
-                parts.append({"ETag": etag, "PartNumber": part_num + 1})
+                parts.append({"ETag": etag, "PartNumber": part_num})
+                part_num += 1
 
+        logger.info("Starting Gantry Ingestion")
         # Submit file completion to gantry API.
-        logger.info("Initializing Upload to Gantry")
-        self.log_store._api_client.request(
+        complete_res = self.log_store._api_client.request(
             "POST",
             "/api/v1/ingest/file",
             json={
@@ -571,15 +293,15 @@ class Gantry:
             },
             headers={"Content-Type": "application/json", "Accept": "application/json"},
         )
-        # batch_id = complete_res["batch_id"]
-        # _batch_success_msg(batch_id, application, self.log_store)
+        batch_id = complete_res["batch_id"]
+        _batch_success_msg(batch_id, application, self.log_store)
 
     @_log_exception
     @typechecked
     def log_records(
         self,
         application: str,
-        version: Optional[Union[int, str, List[Union[str, int]]]] = None,
+        version: Optional[Union[int, str]] = None,
         inputs: Optional[Union[List[dict], pd.DataFrame]] = None,
         outputs: Optional[Union[List[dict], pd.DataFrame]] = None,
         feedback_keys: Optional[List[str]] = None,
@@ -621,8 +343,8 @@ class Gantry:
         Args:
             application (str): Name of the application. Gantry validates and monitors data
                 by function.
-            version (optional, Union[int, str, List[Union[str, int]]]): Version of the function
-                schema to use for validation by Gantry.
+            version (optional, Union[int, str]): Version of the function schema to use for
+                validation by Gantry.
                 If not provided, Gantry will use the latest version. If the version doesn't exist
                 yet, Gantry will create it by auto-generating the schema based on data it has seen.
             inputs (Union[List[dict], pd.Dataframe]): A list of prediction inputs. `inputs[i]`
@@ -656,9 +378,16 @@ class Gantry:
                 in the 'batch' section of the dashboard
 
         """
+        some_preds_exist = _is_not_empty(inputs) or _is_not_empty(outputs)
         preds_exist = _is_not_empty(inputs) and _is_not_empty(outputs)
         feedbacks_exist = _is_not_empty(feedbacks)
 
+        if not preds_exist and some_preds_exist:
+            raise ValueError(
+                "Tried to log records with incomplete prediction "
+                "(both inputs and outputs should be provided)"
+            )
+
         if preds_exist and feedbacks_exist:
             return self._log_predictions_and_feedbacks(
                 application=application,
@@ -871,9 +600,16 @@ class Gantry:
             sample_rate (optional, float): Specify the probability as a float that the event will
                 be sent to Gantry.
         """
+        some_pred_exist = _is_not_empty(inputs) or _is_not_empty(outputs)
         pred_exist = _is_not_empty(inputs) and _is_not_empty(outputs)
         feedback_exist = _is_not_empty(feedback)
 
+        if (not pred_exist) and some_pred_exist:
+            raise ValueError(
+                "Tried to log records with incomplete prediction "
+                "(both inputs and outputs should be provided)"
+            )
+
         if pred_exist and feedback_exist:
             return self._log_prediction_and_feedback_event(
                 application=application,
@@ -953,7 +689,7 @@ class Gantry:
             application,
             feedback_id,  # type: ignore
             feedback,  # type: ignore
-            version,  # type: ignore
+            version,
             timestamp,
         )
 
@@ -968,7 +704,7 @@ class Gantry:
         application: str,
         feedback_id: dict,
         feedback: dict,
-        feedback_version: Optional[int] = None,
+        feedback_version: Optional[Union[str, int]] = None,
         timestamp: Optional[datetime.datetime] = None,
         sample_rate: float = 1.0,
     ):
@@ -996,7 +732,7 @@ class Gantry:
                 the value matches the value in the corresponding prediction. The feedback event
                 is matched to the prediction event using the values of the feedback keys.
             feedback (dict): application feedback object.
-            feedback_version (optional, int): Specify application version.
+            feedback_version (optional, int or str): Specify application version.
             timestamp (optional, datetime.datetime): Record timestamp.
             sample_rate (optional, float): Specify the probability as a float that the event will
                 be sent to Gantry.
@@ -1022,7 +758,7 @@ class Gantry:
         application: str,
         feedback_ids: List[dict],
         feedbacks: Union[List[dict], pd.DataFrame],
-        feedback_version: Optional[int] = None,
+        feedback_version: Optional[Union[str, int]] = None,
         timestamps: Optional[List[datetime.datetime]] = None,
         sort_on_timestamp: bool = False,
         sample_rate: float = 1.0,
@@ -1044,7 +780,7 @@ class Gantry:
                 event using the values of the feedback keys. The i-th feedback_id maps to
                 the i-th feedback.
             feedback (list): List of application feedback objects.
-            feedback_version (optional, int): Specify application version.
+            feedback_version (optional, int or str): Specify application version.
             timestamps (optional, datetime.datetime): Record timestamps. The i-th timestamp
                 maps to the i-th feedback.
             sort_on_timestamp (optional, boolean): Specify whether result should sort on timestamp.
