@@ -82,15 +82,16 @@ def test_create_timestamp_idx(sort, timestamps, record_count, expected):
 
 @freeze_time(CURRENT_TIME)
 @pytest.mark.parametrize("timestamp", [None, SOME_TIME])
-@mock.patch("gantry.logger.client.uuid.uuid4")
-def test_build_feedback_event(mock_uuid, timestamp):
+@pytest.mark.parametrize("feedback_version", [None, 5, "1.2.3"])
+@mock.patch("gantry.logger.event_builder.uuid.uuid4")
+def test_build_feedback_event(mock_uuid, feedback_version, timestamp):
     mock_uuid.return_value = "123abc"
     assert client._build_feedback_event(
         context={"foo": "bar"},
         application="barbaz",
         feedback_id={"a": 10},
         feedback={"b": 20},
-        feedback_version=5,
+        feedback_version=feedback_version,
         timestamp=timestamp,
         batch_id="ABCD1234",
     ) == {
@@ -100,7 +101,7 @@ def test_build_feedback_event(mock_uuid, timestamp):
         "feedback_id": "2a30f5f3b7d1a97cb6132480b992d984",
         "feedback_id_inputs": {"a": 10},
         "log_timestamp": CURRENT_TIME_STR,
-        "metadata": {"feedback_version": 5, "foo": "bar", "func_name": "barbaz"},
+        "metadata": {"feedback_version": feedback_version, "foo": "bar", "func_name": "barbaz"},
         "timestamp": SOME_TIME_STR if timestamp else CURRENT_TIME_STR,
     }
 
@@ -115,7 +116,7 @@ def test_build_feedback_event(mock_uuid, timestamp):
     ],
 )
 @pytest.mark.parametrize("timestamp", [None, SOME_TIME])
-@mock.patch("gantry.logger.client.uuid.uuid4")
+@mock.patch("gantry.logger.event_builder.uuid.uuid4")
 def test_build_prediction_event(mock_uuid, timestamp, feedback_keys, feedback_id):
     mock_uuid.return_value = "123abc"
     assert client._build_prediction_event(
@@ -249,9 +250,7 @@ def test_log_feedback_event_invalid_feedback(feedback, cli_obj, caplog):
     assert "TypeError:" in caplog.text
 
 
-@pytest.mark.parametrize(
-    "feedback_version", ["some-version", b"a-version", 10.5, {"version": "yes"}, [10]]
-)
+@pytest.mark.parametrize("feedback_version", [b"a-version", 10.5, {"version": "yes"}, [10]])
 def test_log_feedback_event_invalid_feedback_version(feedback_version, cli_obj, caplog):
     caplog.set_level(logging.ERROR, logger="gantry.client")
     cli_obj.log_feedback_event(
@@ -297,14 +296,15 @@ def test_log_feedback_event_future_timestamp(timestamp, cli_obj, caplog):
 
 @freeze_time(CURRENT_TIME)
 @pytest.mark.parametrize("timestamp", [None, SOME_TIME])
-@mock.patch("gantry.logger.client.uuid.uuid4")
-def test_log_feedback_event(mock_uuid, timestamp, cli_obj):
+@pytest.mark.parametrize("feedback_version", [None, "1.2.3", 10])
+@mock.patch("gantry.logger.event_builder.uuid.uuid4")
+def test_log_feedback_event(mock_uuid, feedback_version, timestamp, cli_obj):
     mock_uuid.return_value = "12345"
     cli_obj.log_feedback_event(
         application="foobar",
         feedback_id={"A": 100},
         feedback={"value": "bar"},
-        feedback_version=10,
+        feedback_version=feedback_version,
         timestamp=timestamp,
     )
     assert cli_obj.log_store.logs == [
@@ -319,7 +319,7 @@ def test_log_feedback_event(mock_uuid, timestamp, cli_obj):
                     "log_timestamp": CURRENT_TIME_STR,
                     "metadata": {
                         "environment": "test",
-                        "feedback_version": 10,
+                        "feedback_version": feedback_version,
                         "func_name": "foobar",
                     },
                     "timestamp": SOME_TIME_STR if timestamp else CURRENT_TIME_STR,
@@ -331,7 +331,7 @@ def test_log_feedback_event(mock_uuid, timestamp, cli_obj):
 
 
 @pytest.mark.parametrize("timestamp", [None, SOME_TIME])
-@mock.patch("gantry.logger.client.uuid.uuid4")
+@mock.patch("gantry.logger.event_builder.uuid.uuid4")
 def test_log_feedback_event_sample_rate_0_5(mock_uuid, timestamp):
     log_store = mock.MagicMock()
     gantry = client.Gantry(log_store=log_store, environment="test")
@@ -350,7 +350,7 @@ def test_log_feedback_event_sample_rate_0_5(mock_uuid, timestamp):
 
 
 @pytest.mark.parametrize("timestamp", [None, SOME_TIME])
-@mock.patch("gantry.logger.client.uuid.uuid4")
+@mock.patch("gantry.logger.event_builder.uuid.uuid4")
 def test_log_feedback_event_sample_rate_0_5_none(mock_uuid, timestamp):
     log_store = mock.MagicMock()
     gantry = client.Gantry(log_store=log_store, environment="test")
@@ -369,7 +369,7 @@ def test_log_feedback_event_sample_rate_0_5_none(mock_uuid, timestamp):
 
 
 @pytest.mark.parametrize("timestamp", [None, SOME_TIME])
-@mock.patch("gantry.logger.client.uuid.uuid4")
+@mock.patch("gantry.logger.event_builder.uuid.uuid4")
 def test_log_feedback_event_sample_rate_0(mock_uuid, timestamp):
     log_store = mock.MagicMock()
     gantry = client.Gantry(log_store=log_store, environment="test")
@@ -514,7 +514,7 @@ def test_log_prediction_event_invalid_timestamp(timestamp, cli_obj, caplog):
         (None, {"A": 100}),
     ],
 )
-@mock.patch("gantry.logger.client.uuid.uuid4")
+@mock.patch("gantry.logger.event_builder.uuid.uuid4")
 def test_log_prediction_event(mock_uuid, feedback_keys, feedback_id, timestamp, cli_obj):
     mock_uuid.return_value = "12345"
     cli_obj.log_prediction_event(
@@ -562,7 +562,7 @@ def test_log_prediction_event(mock_uuid, feedback_keys, feedback_id, timestamp,
         (None, {"A": 100}),
     ],
 )
-@mock.patch("gantry.logger.client.uuid.uuid4")
+@mock.patch("gantry.logger.event_builder.uuid.uuid4")
 def test_log_prediction_event_sample_rate_0_5(mock_uuid, feedback_keys, feedback_id, timestamp):
     log_store = mock.MagicMock()
     gantry = client.Gantry(log_store=log_store, environment="test")
@@ -592,7 +592,7 @@ def test_log_prediction_event_sample_rate_0_5(mock_uuid, feedback_keys, feedback
         (None, {"A": 100}),
     ],
 )
-@mock.patch("gantry.logger.client.uuid.uuid4")
+@mock.patch("gantry.logger.event_builder.uuid.uuid4")
 def test_log_prediction_event_sample_rate_0_5_none(
     mock_uuid, feedback_keys, feedback_id, timestamp
 ):
@@ -624,7 +624,7 @@ def test_log_prediction_event_sample_rate_0_5_none(
         (None, {"A": 100}),
     ],
 )
-@mock.patch("gantry.logger.client.uuid.uuid4")
+@mock.patch("gantry.logger.event_builder.uuid.uuid4")
 def test_log_prediction_event_sample_rate_0(mock_uuid, feedback_keys, feedback_id, timestamp):
     log_store = mock.MagicMock()
     gantry = client.Gantry(log_store=log_store, environment="test")
@@ -821,7 +821,7 @@ def test_log_record_invalid_timestamp_for_prediction(timestamp, cli_obj, caplog)
 
 @freeze_time(CURRENT_TIME)
 @pytest.mark.parametrize("timestamp", [None, SOME_TIME])
-@mock.patch("gantry.logger.client.uuid.uuid4")
+@mock.patch("gantry.logger.event_builder.uuid.uuid4")
 def test_log_prediction_and_feedback_event(mock_uuid, timestamp, cli_obj):
     mock_uuid.return_value = "12345"
     cli_obj._log_prediction_and_feedback_event(
@@ -864,7 +864,7 @@ def test_log_prediction_and_feedback_event(mock_uuid, timestamp, cli_obj):
 
 
 @pytest.mark.parametrize("timestamp", [None, SOME_TIME])
-@mock.patch("gantry.logger.client.uuid.uuid4")
+@mock.patch("gantry.logger.event_builder.uuid.uuid4")
 def test_log_prediction_and_feedback_event_sample_rate_0_5(mock_uuid, timestamp):
     log_store = mock.MagicMock()
     gantry = client.Gantry(log_store=log_store, environment="test")
@@ -885,7 +885,7 @@ def test_log_prediction_and_feedback_event_sample_rate_0_5(mock_uuid, timestamp)
 
 
 @pytest.mark.parametrize("timestamp", [None, SOME_TIME])
-@mock.patch("gantry.logger.client.uuid.uuid4")
+@mock.patch("gantry.logger.event_builder.uuid.uuid4")
 def test_log_prediction_and_feedback_event_sample_rate_0_5_none(mock_uuid, timestamp):
     log_store = mock.MagicMock()
     gantry = client.Gantry(log_store=log_store, environment="test")
@@ -906,7 +906,7 @@ def test_log_prediction_and_feedback_event_sample_rate_0_5_none(mock_uuid, times
 
 
 @pytest.mark.parametrize("timestamp", [None, SOME_TIME])
-@mock.patch("gantry.logger.client.uuid.uuid4")
+@mock.patch("gantry.logger.event_builder.uuid.uuid4")
 def test_log_prediction_and_feedback_event_sample_rate_0(mock_uuid, timestamp):
     log_store = mock.MagicMock()
     gantry = client.Gantry(log_store=log_store, environment="test")
@@ -971,7 +971,7 @@ def test_log_feedback_invalid_feedback(feedbacks, cli_obj, caplog):
     assert "TypeError:" in caplog.text
 
 
-@pytest.mark.parametrize("feedback_version", ["some-version"])
+@pytest.mark.parametrize("feedback_version", [{"this-is": 10}, ["1.2.3", "2.3.4"]])
 def test_log_feedback_invalid_feedback_version(feedback_version, cli_obj, caplog):
     caplog.set_level(logging.ERROR, logger="gantry.client")
     cli_obj.log_feedback(
@@ -1013,7 +1013,7 @@ def test_log_feedback_invalid_timestamps(timestamps, cli_obj, caplog):
         pd.DataFrame({100: ["some-value", "some-other-value"]}),
     ],
 )
-@mock.patch("gantry.logger.client.uuid.uuid4")
+@mock.patch("gantry.logger.event_builder.uuid.uuid4")
 def test_log_feedback(mock_uuid, feedbacks, random_error, timestamps, sort, cli_obj):
     if random_error:
         mock_uuid.side_effect = RuntimeError
@@ -1076,7 +1076,7 @@ def test_log_feedback(mock_uuid, feedbacks, random_error, timestamps, sort, cli_
         pd.DataFrame({100: ["some-value", "some-other-value"]}),
     ],
 )
-@mock.patch("gantry.logger.client.uuid.uuid4")
+@mock.patch("gantry.logger.event_builder.uuid.uuid4")
 def test_log_feedback_sample_rate_0_5(mock_uuid, feedbacks, timestamps, sort):
     log_store = mock.MagicMock()
     gantry = client.Gantry(log_store=log_store, environment="test")
@@ -1106,7 +1106,7 @@ def test_log_feedback_sample_rate_0_5(mock_uuid, feedbacks, timestamps, sort):
         pd.DataFrame({100: ["some-value", "some-other-value"]}),
     ],
 )
-@mock.patch("gantry.logger.client.uuid.uuid4")
+@mock.patch("gantry.logger.event_builder.uuid.uuid4")
 def test_log_feedback_sample_rate_0_5_none(mock_uuid, feedbacks, timestamps, sort, cli_obj):
     log_store = mock.MagicMock()
     gantry = client.Gantry(log_store=log_store, environment="test")
@@ -1136,7 +1136,7 @@ def test_log_feedback_sample_rate_0_5_none(mock_uuid, feedbacks, timestamps, sor
         pd.DataFrame({100: ["some-value", "some-other-value"]}),
     ],
 )
-@mock.patch("gantry.logger.client.uuid.uuid4")
+@mock.patch("gantry.logger.event_builder.uuid.uuid4")
 def test_log_feedback_sample_rate_0_0(mock_uuid, feedbacks, timestamps, sort):
     log_store = mock.MagicMock()
     gantry = client.Gantry(log_store=log_store, environment="test")
@@ -1340,7 +1340,7 @@ def test_log_predictions_invalid_reference_name(reference_name, cli_obj, caplog)
         pd.DataFrame({"A": [100, 101], "B": [200, 202]}),
     ],
 )
-@mock.patch("gantry.logger.client.uuid.uuid4")
+@mock.patch("gantry.logger.event_builder.uuid.uuid4")
 def test_log_predictions(
     mock_uuid, inputs, outputs, random_error, timestamps, reference_name, sort, cli_obj
 ):
@@ -1437,7 +1437,7 @@ def test_log_predictions(
         pd.DataFrame({"A": [100, 101], "B": [200, 202]}),
     ],
 )
-@mock.patch("gantry.logger.client.uuid.uuid4")
+@mock.patch("gantry.logger.event_builder.uuid.uuid4")
 def test_log_predictions_sample_rate_0_5(
     mock_uuid, inputs, outputs, timestamps, reference_name, sort
 ):
@@ -1479,7 +1479,7 @@ def test_log_predictions_sample_rate_0_5(
         pd.DataFrame({"A": [100, 101], "B": [200, 202]}),
     ],
 )
-@mock.patch("gantry.logger.client.uuid.uuid4")
+@mock.patch("gantry.logger.event_builder.uuid.uuid4")
 def test_log_predictions_sample_rate_0_5_none(
     mock_uuid, inputs, outputs, timestamps, reference_name, sort
 ):
@@ -1521,7 +1521,7 @@ def test_log_predictions_sample_rate_0_5_none(
         pd.DataFrame({"A": [100, 101], "B": [200, 202]}),
     ],
 )
-@mock.patch("gantry.logger.client.uuid.uuid4")
+@mock.patch("gantry.logger.event_builder.uuid.uuid4")
 def test_log_predictions_sample_rate_0(
     mock_uuid, inputs, outputs, timestamps, reference_name, sort
 ):
@@ -1834,7 +1834,7 @@ def test_log_predictions_and_feedbacks_size_mismatch_versions(version, cli_obj,
         pd.DataFrame.from_dict({"A": [100, 101]}),
     ],
 )
-@mock.patch("gantry.logger.client.uuid.uuid4")
+@mock.patch("gantry.logger.event_builder.uuid.uuid4")
 @mock.patch("gantry.logger.client.Gantry._create_batch")
 def test_log_predictions_and_feedbacks_df(
     mock_create_batch,
@@ -1945,6 +1945,7 @@ def test_log_predictions_and_feedbacks_df(
         pd.DataFrame.from_dict({"A": [100, 101]}),
     ],
 )
+@pytest.mark.parametrize("version", [None, 10, "1.2.3"])
 @mock.patch("gantry.logger.client.Gantry._log_predictions_and_feedbacks")
 @mock.patch("gantry.logger.client.Gantry.log_predictions")
 @mock.patch("gantry.logger.client.Gantry.log_feedback")
@@ -1952,6 +1953,7 @@ def test_log_records_preds_and_feedback(
     mock_feedback,
     mock_preds,
     mock_preds_and_feedback,
+    version,
     test_inputs,
     test_outputs,
     test_feedbacks,
@@ -1959,7 +1961,7 @@ def test_log_records_preds_and_feedback(
 ):
     cli_obj.log_records(
         application="foobar",
-        version="2.0.1",
+        version=version,
         inputs=test_inputs,
         outputs=test_outputs,
         feedback_keys=["A"],
@@ -1995,14 +1997,15 @@ def test_log_records_preds_and_feedback(
         pd.DataFrame.from_dict({"A": [100, 101]}),
     ],
 )
-def test_log_records_preds_and_feedback_0_5(test_inputs, test_outputs, test_feedbacks):
+@pytest.mark.parametrize("version", [None, 10, "1.2.3"])
+def test_log_records_preds_and_feedback_0_5(version, test_inputs, test_outputs, test_feedbacks):
     log_store = mock.MagicMock()
     gantry = client.Gantry(log_store=log_store, environment="test")
 
     with mock.patch("random.random", return_value=0.4):
         gantry.log_records(
             application="foobar",
-            version="2.0.1",
+            version=version,
             inputs=test_inputs,
             outputs=test_outputs,
             feedback_keys=["A"],
@@ -2038,14 +2041,17 @@ def test_log_records_preds_and_feedback_0_5(test_inputs, test_outputs, test_feed
         pd.DataFrame.from_dict({"A": [100, 101]}),
     ],
 )
-def test_log_records_preds_and_feedback_0_5_none(test_inputs, test_outputs, test_feedbacks):
+@pytest.mark.parametrize("version", [None, 10, "1.2.3"])
+def test_log_records_preds_and_feedback_0_5_none(
+    version, test_inputs, test_outputs, test_feedbacks
+):
     log_store = mock.MagicMock()
     gantry = client.Gantry(log_store=log_store, environment="test")
 
     with mock.patch("random.random", return_value=0.6):
         gantry.log_records(
             application="foobar",
-            version="2.0.1",
+            version=version,
             inputs=test_inputs,
             outputs=test_outputs,
             feedback_keys=["A"],
@@ -2081,14 +2087,15 @@ def test_log_records_preds_and_feedback_0_5_none(test_inputs, test_outputs, test
         pd.DataFrame.from_dict({"A": [100, 101]}),
     ],
 )
-def test_log_records_preds_and_feedback_0_0(test_inputs, test_outputs, test_feedbacks):
+@pytest.mark.parametrize("version", [None, 10, "1.2.3"])
+def test_log_records_preds_and_feedback_0_0(version, test_inputs, test_outputs, test_feedbacks):
     log_store = mock.MagicMock()
     gantry = client.Gantry(log_store=log_store, environment="test")
 
     with mock.patch("random.random", return_value=0.6):
         gantry.log_records(
             application="foobar",
-            version="2.0.1",
+            version=version,
             inputs=test_inputs,
             outputs=test_outputs,
             feedback_keys=["A"],
@@ -2117,15 +2124,16 @@ def test_log_records_preds_and_feedback_0_0(test_inputs, test_outputs, test_feed
         pd.DataFrame.from_dict({"A": [100, 101]}),
     ],
 )
+@pytest.mark.parametrize("version", [None, 10, "1.2.3"])
 @mock.patch("gantry.logger.client.Gantry._log_predictions_and_feedbacks")
 @mock.patch("gantry.logger.client.Gantry.log_predictions")
 @mock.patch("gantry.logger.client.Gantry.log_feedback")
 def test_log_records_preds_only(
-    mock_feedback, mock_preds, mock_preds_and_feedback, test_inputs, test_outputs, cli_obj
+    mock_feedback, mock_preds, mock_preds_and_feedback, version, test_inputs, test_outputs, cli_obj
 ):
     cli_obj.log_records(
         application="foobar",
-        version="2.0.1",
+        version=version,
         inputs=test_inputs,
         outputs=test_outputs,
         feedback_keys=["A"],
@@ -2147,15 +2155,16 @@ def test_log_records_preds_only(
         pd.DataFrame.from_dict({"A": [200, 201]}),
     ],
 )
+@pytest.mark.parametrize("version", [None, 10, "1.2.3"])
 @mock.patch("gantry.logger.client.Gantry._log_predictions_and_feedbacks")
 @mock.patch("gantry.logger.client.Gantry.log_predictions")
 @mock.patch("gantry.logger.client.Gantry.log_feedback")
 def test_log_records_feedback_only(
-    mock_feedback, mock_preds, mock_preds_and_feedback, test_feedbacks, cli_obj
+    mock_feedback, mock_preds, mock_preds_and_feedback, version, test_feedbacks, cli_obj
 ):
     cli_obj.log_records(
         application="foobar",
-        version="2.0.1",
+        version=version,
         inputs=None,
         outputs=None,
         feedback_keys=["A"],
@@ -2182,6 +2191,7 @@ def test_log_records_feedback_only(
     "test_inputs",
     [None, pd.DataFrame(), []],
 )
+@pytest.mark.parametrize("version", [None, 10, "1.2.3"])
 @mock.patch("gantry.logger.client.Gantry._log_predictions_and_feedbacks")
 @mock.patch("gantry.logger.client.Gantry.log_predictions")
 @mock.patch("gantry.logger.client.Gantry.log_feedback")
@@ -2189,6 +2199,7 @@ def test_log_records_no_data(
     mock_feedback,
     mock_preds,
     mock_preds_and_feedback,
+    version,
     test_inputs,
     test_outputs,
     test_feedbacks,
@@ -2196,7 +2207,7 @@ def test_log_records_no_data(
 ):
     cli_obj.log_records(
         application="foobar",
-        version="2.0.1",
+        version=version,
         inputs=test_inputs,
         outputs=test_outputs,
         feedback_keys=["A"],
@@ -2211,6 +2222,37 @@ def test_log_records_no_data(
     mock_feedback.assert_not_called()
 
 
+@pytest.mark.parametrize(
+    "test_feedbacks",
+    [None, pd.DataFrame(), [{"A": 3}, {"A": 3}]],
+)
+@mock.patch("gantry.logger.client.Gantry._log_predictions_and_feedbacks")
+@mock.patch("gantry.logger.client.Gantry.log_predictions")
+@mock.patch("gantry.logger.client.Gantry.log_feedback")
+def test_log_records_incomplete_preds(
+    mock_feedback,
+    mock_preds,
+    mock_preds_and_feedback,
+    test_feedbacks,
+    cli_obj,
+):
+    cli_obj.log_records(
+        application="foobar",
+        version="2.0.1",
+        inputs=[],
+        outputs=[{"A": 1}, {"A": 2}],
+        feedback_keys=["A"],
+        feedbacks=test_feedbacks,
+        timestamps=None,
+        reference_name="some-tag",
+        sort_on_timestamp=False,
+    )
+
+    mock_preds_and_feedback.assert_not_called()
+    mock_preds.assert_not_called()
+    mock_feedback.assert_not_called()
+
+
 @mock.patch("gantry.logger.client.Gantry._log_prediction_and_feedback_event")
 @mock.patch("gantry.logger.client.Gantry.log_prediction_event")
 @mock.patch("gantry.logger.client.Gantry.log_feedback_event")
@@ -2332,3 +2374,44 @@ def test_log_record_no_data(mock_feedback, mock_pred, mock_pred_and_feedback, te
     mock_pred_and_feedback.assert_not_called()
     mock_pred.assert_not_called()
     mock_feedback.assert_not_called()
+
+
+@pytest.mark.parametrize("feedback", [None, {}, {"B": 200}])
+@mock.patch("gantry.logger.client.Gantry._log_prediction_and_feedback_event")
+@mock.patch("gantry.logger.client.Gantry.log_prediction_event")
+@mock.patch("gantry.logger.client.Gantry.log_feedback_event")
+def test_log_record_incomplete_preds(
+    mock_feedback, mock_pred, mock_pred_and_feedback, feedback, cli_obj
+):
+    cli_obj.log_record(
+        application="foobar",
+        version="2.0.1",
+        inputs={},
+        outputs={"B": 100},
+        feedback_keys=["A"],
+        feedback=feedback,
+        timestamp=None,
+    )
+
+    mock_pred_and_feedback.assert_not_called()
+    mock_pred.assert_not_called()
+    mock_feedback.assert_not_called()
+
+
+@pytest.mark.parametrize(
+    "test_inputs",
+    [[{"A": 100}, {"A": 101}]],
+)
+@mock.patch("gantry.logger.utils.get_file_linecount")
+def test_log_file(mock_line_count, test_inputs):
+    log_store = mock.MagicMock()
+    gantry = client.Gantry(log_store=log_store, environment="test")
+    mock_line_count.return_value = 5
+
+    gantry.log_file(
+        "foobar",
+        "dir/csv_with_headers.csv",
+        version="2.0.1",
+    )
+
+    log_store.log_batch.assert_not_called()
